#
#  Copyright (c) 2022, salesforce.com, inc.
#  All rights reserved.
#  SPDX-License-Identifier: BSD-3-Clause
#  For full license text, see the LICENSE file in the repo root or https://opensource.org/licenses/BSD-3-Clause
#

API_VERSION_V1 = "v1"
API_VERSION_V2 = "v2"

AUTH_PARAM_GRANT_TYPE = 'grant_type'
AUTH_PARAM_P_D = 'password'
AUTH_PARAM_CLIENT_ID = 'client_id'
AUTH_PARAM_CLIENT_SECRET = 'client_secret'
AUTH_PARAM_USERNAME = 'username'
AUTH_RESPONSE_ACCESS_TOKEN = 'access_token'
AUTH_RESPONSE_EXPIRES_IN = 'expires_in'
AUTH_RESPONSE_INSTANCE_URL = 'instance_url'

AUTH_PARAM_CDP_GRANT_TYPE = 'urn:salesforce:grant-type:external:cdp'
AUTH_PARAM_CDP_SUBJECT_TOKEN = 'subject_token'
AUTH_PARAM_CDP_SUBJECT_TOKEN_TYPE = 'subject_token_type'
AUTH_PARAM_CDP_SUBJECT_TOKEN_TYPE_VALUE = 'urn:ietf:params:oauth:token-type:access_token'
AUTH_PARAM_CDP_SUBJECT_TOKEN_TYPE_VALUE = 'urn:ietf:params:oauth:token-type:access_token'
AUTH_PARAM_REFRESH_TOKEN_GRANT_TYPE = 'refresh_token'

DATA_TYPE_TIMESTAMP = 'TIMESTAMP'

ENCODING_ASCII = 'ascii'

QUERY_RESPONSE_KEY_ARROW_STREAM = 'arrowStream'
QUERY_RESPONSE_KEY_DATA = 'data'
QUERY_RESPONSE_KEY_METADATA = 'metadata'
QUERY_RESPONSE_KEY_METADATA_TYPE = 'type'
QUERY_RESPONSE_KEY_DONE = 'done'
QUERY_RESPONSE_KEY_NEXT_BATCH_ID = 'nextBatchId'
QUERY_RESPONSE_KEY_PLACE_IN_ORDER = 'placeInOrder'

QUERY_HEADER_KEY_AUTHORIZATION = 'Authorization'
QUERY_HEADER_KEY_CONTENT_TYPE = 'Content-Type'
QUERY_HEADER_KEY_ACCEPT_ENCODING = 'Accept-Encoding'
QUERY_HEADER_VALUE_APPLICATION_JSON = 'application/json'
QUERY_HEADER_VALUE_GZIP = 'gzip'